package uk.ac.umist.co.brailletrans.tests;
import uk.ac.umist.co.brailletrans.*;
import uk.ac.umist.co.brailletrans.utils.*;

class SmallSpeedTest extends Translator
{
  public static void main(String[] args) throws Exception
  {
    int numberTests = 0;       // number of tests to run
    String inFilename = "";    // filename of text file to translate
    String languageName = ""; // filename of language file to use
    int state = 1;             // the state of the virtual machine
    Stopwatch loopTimer;
    Stopwatch runTimer;
    String translationType;

    // Read the input from the user on the command line
    if (args.length != 5)
    {
      System.out.println("\nSmallSpeedTest USAGE:\n javaSmallSpeedTest <from> <language> <state> <number tests> <type>");
      System.out.println("\ntype=[TUNI|TINT|T256|TC|TDUMMYC|TUNIFAST]");
      System.exit(SUCCESS);
    }

    inFilename = new String(args[0]);
    languageName = new String(args[1]);
    state = (Integer.parseInt(args[2]));
    numberTests = (Integer.parseInt(args[3]));
    translationType = new String(args[4]);

    loopTimer = new Stopwatch(numberTests);
    runTimer = new Stopwatch(numberTests);

    if (translationType.equalsIgnoreCase("t256"))
    {
      // Get input files to translate
      int[] toConvertIntArray = null;
      toConvertIntArray = Translator.readIntArrayFromDisk(inFilename);
      // Initialise
      Language256 language = new Language256(languageName);
      // Set
      language.setState(state);
      // Run
      runTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        language.translate(toConvertIntArray);
      }
      runTimer.stop();
      loopTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        ;
      }
      loopTimer.stop();
    }
    else if (translationType.equalsIgnoreCase("tint"))
    {
      // Get input files to translate
      int[] toConvertIntArray = Translator.readIntArrayFromDisk(inFilename);
      // Initialise
      LanguageInteger language = new LanguageInteger(languageName);
      // Set
      language.setState(state);
      // Run
      runTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        language.translate(toConvertIntArray);
      }
      runTimer.stop();
      loopTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        ;
      }
      loopTimer.stop();
    }
    else if (translationType.equalsIgnoreCase("tc"))
    {
      // Get byte version
      byte[] toConvertByteArray = Translator.turnIntoByteArray(readIntArrayFromDisk(inFilename));
      byte cstate = (byte) state;
      TranslateC language = new TranslateC();
      languageName = languageName + Language.FILE_EXTENSION_DELIMITER
        + Language256.FILENAME_EXTENSION;
      byte[] output = new byte[(int) (toConvertByteArray.length * Language256.MAX_COMPRESS)];
      // Initialise
      language.InitBrl(languageName);
      // Set
      language.SetState(cstate);
      // Run
      runTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        language.Translate(toConvertByteArray, output);
      }
      runTimer.stop();
      loopTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        ;
      }
      loopTimer.stop();

    }
    else if (translationType.equalsIgnoreCase("tdummyc"))
    {
      // Get byte version
      byte[] toConvertByteArray = Translator.turnIntoByteArray(readIntArrayFromDisk(inFilename));
      byte cstate = (byte) state;
      TranslateCDummy language = new TranslateCDummy();
      byte[] output = new byte[(int) (toConvertByteArray.length * Language256.MAX_COMPRESS)];
      // Initialise
      language.InitBrl(languageName);
      // Set
      language.SetState(cstate);
      // Run
      runTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        language.Translate(toConvertByteArray, output);
      }
      runTimer.stop();
      loopTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        ;
      }
      loopTimer.stop();
    }
    else if (translationType.equalsIgnoreCase("tuni"))
    {
      // Get String version
      String toConvertString = Translator.readStringFromDisk(inFilename);
      // Initialise
      LanguageUnicode language = new LanguageUnicode(languageName);
      // Set
      language.setState(state);
      // Run
      runTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        language.translate(toConvertString);
      }
      runTimer.stop();
      loopTimer.start();
      for (int i = 0; i < numberTests; i++)
      {
        ;
      }
      loopTimer.stop();

    }
    else
    {
      System.out.println("Not a valid language type.");
    }

    // Display results
    System.out.println("State: " + state);
    System.out.println("Language used: " + languageName);
    System.out.println("Translation type: " + translationType);
    System.out.println("File translated: " + inFilename);
    System.out.println("Number of runs: " + numberTests);
    System.out.println("Totals \treal\t" + runTimer.getTotal() +
      "\tloop\t" + loopTimer.getTotal());
    System.out.println("Means \treal\t" + runTimer.meanTime() +
      "\tloop\t" + loopTimer.meanTime());
    System.out.println("SDs \treal\t" + runTimer.sdTime() +
      "\tloop\t" + loopTimer.sdTime());

  }

}
