package uk.ac.umist.co.brailletrans.tests;
import uk.ac.umist.co.brailletrans.*;
import uk.ac.umist.co.brailletrans.utils.*;

class SpeedTest extends Translator
{
  public static void main(String[] args) throws Exception
  {
    int numberTests = 0;       // number of tests to run
    String inFilename = "";    // filename of text file to translate
    String languageName = ""; // filename of language file to use
    int state = 1;             // the state of the virtual machine
    Stopwatch loadTimer;
    Stopwatch runTimer;
    String translationType;

    // Read the input from the user on the command line
    if (args.length != 5)
    {
      System.out.println("\nTestSpeed USAGE:\n java brailleTrans.TestSpeed <from> <language> <state> <number tests> <type>");
      System.out.println("\ntype=[TUNI|TINT|T256|TC|TDUMMYC|TUNIFAST]");
      System.exit(SUCCESS);
    }

    inFilename = new String(args[0]);
    languageName = new String(args[1]);
    state = (Integer.parseInt(args[2]));
    numberTests = (Integer.parseInt(args[3]));
    translationType = new String(args[4]);

    loadTimer = new Stopwatch(numberTests);
    runTimer = new Stopwatch(numberTests);

    if (translationType.equalsIgnoreCase("t256"))
    {
      // Get input files to translate
      int[] toConvertIntArray = null;
      toConvertIntArray = Translator.readIntArrayFromDisk(inFilename);
      for (int i = 0; i < numberTests; i++)
      {
        // Initialise
        loadTimer.start();
        Language256 language = new Language256(languageName);
        loadTimer.stop();
        // Set
        language.setState(state);
        // Run
        runTimer.start();
        language.translate(toConvertIntArray);
        runTimer.stop();
        System.out.println("Done " + (i+1) + " of " + numberTests);
      }

    }
    else if (translationType.equalsIgnoreCase("tint"))
    {
      // Get input files to translate
      int[] toConvertIntArray = Translator.readIntArrayFromDisk(inFilename);
      for (int i = 0; i < numberTests; i++)
      {
        // Initialise
        loadTimer.start();
        LanguageInteger language = new LanguageInteger(languageName);
        loadTimer.stop();
        // Set
        language.setState(state);
        // Run
        runTimer.start();
        language.translate(toConvertIntArray);
        runTimer.stop();
        System.out.println("Done " + (i + 1) + " of " + numberTests);
      }
    }
    else if (translationType.equalsIgnoreCase("tc"))
    {
      // Get byte version
      byte[] toConvertByteArray = Translator.turnIntoByteArray(readIntArrayFromDisk(inFilename));
      byte cstate = (byte) state;
      TranslateC language = new TranslateC();
      languageName = languageName + Language.FILE_EXTENSION_DELIMITER
        + Language256.FILENAME_EXTENSION;
      for (int i = 0; i < numberTests; i++)
      {
        byte[] output = new byte[(int) (toConvertByteArray.length * Language256.MAX_COMPRESS)];
        // Initialise
        loadTimer.start();
        language.InitBrl(languageName);
        loadTimer.stop();
        // Set
        language.SetState(cstate);
        // Run
        runTimer.start();
        language.Translate(toConvertByteArray, output);
        runTimer.stop();
        System.out.println("Done " + (i + 1) + " of " + numberTests);
      }
    }
    else if (translationType.equalsIgnoreCase("tdummyc"))
    {
      // Get byte version
      byte[] toConvertByteArray = Translator.turnIntoByteArray(readIntArrayFromDisk(inFilename));
      byte cstate = (byte) state;
      TranslateCDummy language = new TranslateCDummy();
      for (int i = 0; i < numberTests; i++)
      {
        byte[] output = new byte[(int) (toConvertByteArray.length * Language256.MAX_COMPRESS)];
        // Initialise
        loadTimer.start();
        language.InitBrl(languageName);
        loadTimer.stop();
        // Set
        language.SetState(cstate);
        // Run
        runTimer.start();
        language.Translate(toConvertByteArray, output);
        runTimer.stop();
        System.out.println("Done " + (i + 1) + " of " + numberTests);
      }

    }
    else if (translationType.equalsIgnoreCase("tuni"))
    {
      // Get String version
      String toConvertString = Translator.readStringFromDisk(inFilename);
      for (int i = 0; i < numberTests; i++)
      {
        // Initialise
        loadTimer.start();
        LanguageUnicode language = new LanguageUnicode(languageName);
        loadTimer.stop();
        // Set
        language.setState(state);
        // Run
        runTimer.start();
        language.translate(toConvertString);
        runTimer.stop();
        System.out.println("Done " + (i + 1) + " of " + numberTests);
      }
    }
    else
    {
      System.out.println("Not a valid language type.");
    }

    // Display results
    System.out.println("State: " + state);
    System.out.println("Language used: " + languageName);
    System.out.println("Translation type: " + translationType);
    System.out.println("File translated: " + inFilename);
    System.out.println("Number of runs: " + numberTests);
    System.out.println("Totals \trun\t" + runTimer.getTotal() +
      "\tload\t" + loadTimer.getTotal());
    System.out.println("Means \trun\t" + runTimer.meanTime() +
      "\tload\t" + loadTimer.meanTime());
    System.out.println("SDs \trun\t" + runTimer.sdTime() +
      "\tload\t" + loadTimer.sdTime());

  }

}